@@ -2,10 +2,12 @@
 # django false-positives:
 # pylint: disable=unexpected-keyword-arg,no-value-for-parameter
 from __future__ import unicode_literals
+from __future__ import absolute_import
 
 import os
 import stat
 import glob
+import fnmatch
 import logging
 import tarfile
 import socket
@@ -30,18 +32,26 @@ class Changes(debian.deb822.Changes):
     TYPE_DEFAULT = 0
     TYPE_BREQ = 1
     TYPE_BRES = 2
-    TYPES = {TYPE_DEFAULT: "",
-             TYPE_BREQ: "_mini-buildd-buildrequest",
-             TYPE_BRES: "_mini-buildd-buildresult"}
+    TYPE2FILENAME_ID = {TYPE_DEFAULT: "",
+                        TYPE_BREQ: "_mini-buildd-buildrequest",
+                        TYPE_BRES: "_mini-buildd-buildresult"}
 
-    BUILDREQUEST_RE = re.compile("^.+" + TYPES[TYPE_BREQ] + "_[^_]+.changes$")
-    BUILDRESULT_RE = re.compile("^.+" + TYPES[TYPE_BRES] + "_[^_]+.changes$")
+    TYPE2NAME = {TYPE_DEFAULT: "upload",
+                 TYPE_BREQ: "buildrequest",
+                 TYPE_BRES: "buildresult"}
+
+    BUILDREQUEST_RE = re.compile("^.+" + TYPE2FILENAME_ID[TYPE_BREQ] + "_[^_]+.changes$")
+    BUILDRESULT_RE = re.compile("^.+" + TYPE2FILENAME_ID[TYPE_BRES] + "_[^_]+.changes$")
+
+    def _spool_hash_from_file(self):
+        return None if not os.path.exists(self._file_path) else mini_buildd.misc.sha1_of_file(self._file_path)
 
     def __init__(self, file_path):
         self._file_path = file_path
         self._file_name = os.path.basename(file_path)
         self._new = not os.path.exists(file_path)
-        self._sha1 = None if self._new else mini_buildd.misc.sha1_of_file(file_path)
+        # Instance might be produced from a temporary file, so we need to save the hash now.
+        self._spool_hash = self._spool_hash_from_file()
 
         if self.BUILDREQUEST_RE.match(self._file_name):
             self._type = self.TYPE_BREQ
@@ -56,6 +66,7 @@ class Changes(debian.deb822.Changes):
 
         # This is just for stat/display purposes
         self.remote_http_url = None
+        self.live_buildlog_url = None
 
     def __unicode__(self):
         if self.type == self.TYPE_BREQ:
@@ -89,7 +100,7 @@ class Changes(debian.deb822.Changes):
         return "{p}_{v}{x}_{a}.changes".format(p=package,
                                                v=mini_buildd.misc.strip_epoch(version),
                                                a=arch,
-                                               x=cls.TYPES[mbd_type])
+                                               x=cls.TYPE2FILENAME_ID[mbd_type])
 
     def gen_file_name(self, arch, mbd_type):
         return self.gen_changes_file_name(self["Source"], self["Version"], arch, mbd_type)
@@ -122,6 +133,16 @@ class Changes(debian.deb822.Changes):
     def buildlog_name(self):
         return "{s}_{v}_{a}.buildlog".format(s=self["Source"], v=self["Version"], a=self["Architecture"])
 
+    @property
+    def live_buildlog_name(self):
+        return "{spool_id}.buildlog".format(spool_id=self.get_spool_id())
+
+    def get_live_buildlog_loc(self):
+        return "/mini_buildd/live-buildlogs/{logfile}".format(logfile=self.live_buildlog_name)
+
+    def get_live_buildlog_url(self, base_url):
+        return "{base_url}/{loc}".format(base_url=base_url, loc=self.get_live_buildlog_loc())
+
     def get_pkglog_dir(self, installed, relative=True):
         """
         Package log path for this changes file: REPOID/[_failed]/PACKAGE/VERSION/ARCH
@@ -172,8 +193,11 @@ class Changes(debian.deb822.Changes):
     def magic_backport_mode(self):
         return bool(re.search(r"\*\s*MINI_BUILDD:\s*BACKPORT_MODE", self._magic_get_changes()))
 
+    def get_spool_id(self):
+        return "{type}-{hash}".format(type=self.TYPE2NAME[self._type], hash=self._spool_hash)
+
     def get_spool_dir(self):
-        return os.path.join(mini_buildd.setup.SPOOL_DIR, self._sha1)
+        return os.path.join(mini_buildd.setup.SPOOL_DIR, self.get_spool_id())
 
     def get_pkg_id(self, with_arch=False, arch_separator=":"):
         pkg_id = "{s}_{v}".format(s=self["Source"], v=self["Version"])
@@ -210,7 +234,7 @@ class Changes(debian.deb822.Changes):
             LOG.info("Signing changes: {f}".format(f=self._file_path))
             if gnupg:
                 gnupg.sign(self._file_path)
-            self._sha1 = mini_buildd.misc.sha1_of_file(self._file_path)
+            self._spool_hash = self._spool_hash_from_file()
         except:
             # Existence of the file name is used as flag
             if os.path.exists(self._file_path):
@@ -265,17 +289,22 @@ class Changes(debian.deb822.Changes):
         for _load, remote in sorted(remotes.items()):
             try:
                 self.upload(mini_buildd.misc.HoPo(remote.ftp))
-                self.remote_http_url = "http://{r}".format(r=remote.http)
+                remote_url = "http://" + remote.http
+                self.remote_http_url = remote_url
+                self.live_buildlog_url = self.get_live_buildlog_url(base_url=remote_url)
                 return
             except Exception as e:
                 mini_buildd.setup.log_exception(LOG, "Uploading to '{h}' failed".format(h=remote.ftp), e, logging.WARNING)
 
         raise Exception("Buildrequest upload failed for {a}/{c}".format(a=arch, c=codename))
 
-    def tar(self, tar_path, add_files=None):
+    def tar(self, tar_path, add_files=None, exclude_glob=None):
         with contextlib.closing(tarfile.open(tar_path, "w")) as tar:
             def tar_add(file_name):
-                tar.add(file_name, arcname=os.path.basename(file_name))
+                if exclude_glob and fnmatch.fnmatch(file_name, exclude_glob):
+                    LOG.info("Excluding \"{f}\" from tar archive \"{tar}\".".format(f=file_name, tar=tar_path))
+                else:
+                    tar.add(file_name, arcname=os.path.basename(file_name))
 
             tar_add(self._file_path)
             for f in self.get_files():
@@ -373,7 +402,8 @@ class Changes(debian.deb822.Changes):
                                     os.path.join(path, "apt_preferences"),
                                     os.path.join(path, "apt_keys"),
                                     chroot_setup_script,
-                                    os.path.join(path, "sbuildrc_snippet")] + files_from_pool)
+                                    os.path.join(path, "sbuildrc_snippet")] + files_from_pool,
+                         exclude_glob="*.deb")
                 breq.add_file(breq.file_path + ".tar")
 
                 breq["Upload-Result-To"] = daemon.mbd_get_ftp_hopo().string
