@@ -1,14 +1,13 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+from __future__ import absolute_import
 
 import os
 import re
 import time
 import shutil
 import glob
-import tempfile
 import threading
-import subprocess
 import Queue
 import collections
 import urllib2
@@ -21,6 +20,7 @@ import debian.debian_support
 import django.conf
 
 import mini_buildd.misc
+import mini_buildd.call
 import mini_buildd.changes
 import mini_buildd.gnupg
 import mini_buildd.api
@@ -204,7 +204,7 @@ class KeyringPackage(mini_buildd.misc.TmpDir):
         LOG.debug("KeyringPackage using key: '{i}'".format(i=self.key_id))
 
         self.package_name = "{i}-archive-keyring".format(i=identity)
-        self.environment = mini_buildd.misc.taint_env(
+        self.environment = mini_buildd.call.taint_env(
             {"DEBEMAIL": debemail,
              "DEBFULLNAME": debfullname,
              "GNUPGHOME": gpg.home})
@@ -245,19 +245,19 @@ class KeyringPackage(mini_buildd.misc.TmpDir):
                             mini_buildd.misc.open_utf8(os.path.join(p, file_name), "w").write(apt_line + "\n")
 
         # Generate changelog entry
-        mini_buildd.misc.call(["debchange",
+        mini_buildd.call.Call(["debchange",
                                "--create",
-                               "--package={p}".format(p=self.package_name),
-                               "--newversion={v}".format(v=self.version),
+                               "--package", self.package_name,
+                               "--newversion", self.version,
                                "Automatic keyring package for archive '{i}'.".format(i=identity)],
                               cwd=p,
-                              env=self.environment)
+                              env=self.environment).log().check()
 
-        mini_buildd.misc.call(["dpkg-source",
+        mini_buildd.call.Call(["dpkg-source",
                                "-b",
                                "package"],
                               cwd=self.tmpdir,
-                              env=self.environment)
+                              env=self.environment).log().check()
 
         # Compute DSC file name
         self.dsc = os.path.join(self.tmpdir,
@@ -272,11 +272,11 @@ class DSTPackage(mini_buildd.misc.TmpDir):
         dst_dir = os.path.join(self.tmpdir, "package")
         shutil.copytree(tpl_dir, dst_dir)
         if version:
-            mini_buildd.misc.call(["debchange",
-                                   "--newversion={v}".format(v=version),
+            mini_buildd.call.Call(["debchange",
+                                   "--newversion", version,
                                    "Version update '{v}'.".format(v=version)],
-                                  cwd=dst_dir)
-        mini_buildd.misc.call(["dpkg-source", "-b", "package"], cwd=self.tmpdir)
+                                  cwd=dst_dir).log().check()
+        mini_buildd.call.Call(["dpkg-source", "-b", "package"], cwd=self.tmpdir).log().check()
         self.dsc = glob.glob(os.path.join(self.tmpdir, "*.dsc"))[0]
 
 
@@ -625,29 +625,29 @@ class Daemon(object):
     def _port(self, dsc_url, package, dist, version, extra_cl_entries=None):
         t = mini_buildd.misc.TmpDir()
         try:
-            env = mini_buildd.misc.taint_env({"DEBEMAIL": self.model.email_address,
+            env = mini_buildd.call.taint_env({"DEBEMAIL": self.model.email_address,
                                               "DEBFULLNAME": self.model.mbd_fullname,
                                               "GNUPGHOME": self.model.mbd_gnupg.home})
 
             # Download DSC via dget.
-            mini_buildd.misc.call(["dget",
+            mini_buildd.call.Call(["dget",
                                    "--allow-unauthenticated",
                                    "--download-only",
                                    dsc_url],
                                   cwd=t.tmpdir,
-                                  env=env)
+                                  env=env).log().check()
 
             # Get SHA1 of original dsc file
             original_dsc_sha1sum = mini_buildd.misc.sha1_of_file(os.path.join(t.tmpdir, os.path.basename(dsc_url)))
 
             # Unpack DSC (note: dget does not support -x to a dedcicated dir).
             dst = "debian_source_tree"
-            mini_buildd.misc.call(["dpkg-source",
+            mini_buildd.call.Call(["dpkg-source",
                                    "-x",
                                    os.path.basename(dsc_url),
                                    dst],
                                   cwd=t.tmpdir,
-                                  env=env)
+                                  env=env).log().check()
 
             dst_path = os.path.join(t.tmpdir, dst)
 
@@ -657,25 +657,25 @@ class Daemon(object):
             LOG.debug("Port: Found original version/author: {v}/{a}".format(v=original_version, a=original_author))
 
             # Change changelog in DST
-            mini_buildd.misc.call(["debchange",
-                                   "--newversion={v}".format(v=version),
+            mini_buildd.call.Call(["debchange",
+                                   "--newversion", version,
                                    "--force-distribution",
                                    "--force-bad-version",
                                    "--preserve",
-                                   "--dist={d}".format(d=dist),
+                                   "--dist", dist,
                                    "Automated port via mini-buildd (no changes). Original DSC's SHA1: {s}.".format(s=original_dsc_sha1sum)],
                                   cwd=dst_path,
-                                  env=env)
+                                  env=env).log().check()
 
             for entry in (extra_cl_entries or []) + ["MINI_BUILDD: BACKPORT_MODE"]:
-                mini_buildd.misc.call(["debchange",
+                mini_buildd.call.Call(["debchange",
                                        "--append",
                                        entry],
                                       cwd=dst_path,
-                                      env=env)
+                                      env=env).log().check()
 
             # Repack DST
-            mini_buildd.misc.call(["dpkg-source", "-b", dst], cwd=t.tmpdir, env=env)
+            mini_buildd.call.Call(["dpkg-source", "-b", dst], cwd=t.tmpdir, env=env).log().check()
             dsc = os.path.join(t.tmpdir,
                                mini_buildd.changes.Changes.gen_dsc_file_name(package,
                                                                              version))
@@ -688,18 +688,16 @@ class Daemon(object):
                                                                                      "source"))
 
             # Generate Changes file
-            with tempfile.TemporaryFile() as err:
-                with mini_buildd.misc.open_utf8(changes, "w") as out:
-                    subprocess.check_call(["dpkg-genchanges",
-                                           "-S",
-                                           "-sa",
-                                           "-v{v}".format(v=original_version),
-                                           "-DX-Mini-Buildd-Originally-Changed-By={a}".format(a=original_author).encode(mini_buildd.setup.CHAR_ENCODING)],
-                                          cwd=dst_path,
-                                          env=env,
-                                          stdout=out,
-                                          stderr=err)
-                    mini_buildd.misc.log_call_output(LOG.warn, "dpkg-genchanges warning:", err)
+            with open(changes, "w+") as out:
+                # Note: dpkg-genchanges has a home-brewed options parser. It does not allow, for example, "-v 1.2.3", only "-v1.2.3", so we need to use *one* sequence item fo that.
+                mini_buildd.call.Call(["dpkg-genchanges",
+                                       "-S",
+                                       "-sa",
+                                       "-v{v}".format(v=original_version),
+                                       "-DX-Mini-Buildd-Originally-Changed-By={a}".format(a=original_author).encode(mini_buildd.setup.CHAR_ENCODING)],
+                                      cwd=dst_path,
+                                      env=env,
+                                      stdout=out).log().check()
 
             # Sign and add to incoming queue
             self.model.mbd_gnupg.sign(changes)
