@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+from __future__ import absolute_import
 
 import os
 import platform
@@ -9,7 +10,6 @@ import shutil
 import codecs
 import glob
 import errno
-import subprocess
 import threading
 import socket
 import Queue
@@ -410,6 +410,10 @@ def guess_default_dirchroot_backend(overlay, aufs):
     return overlay
 
 
+def chroot_libdir_path(codename, architecture):
+    return os.path.join(mini_buildd.setup.CHROOTS_LIBDIR, codename, architecture)
+
+
 def pkg_fmt(status, distribution, package, version, extra=None, message=None):
     "Generate a package status line."
     fmt = "{status} ({distribution}): {package} {version}".format(status=status,
@@ -524,7 +528,7 @@ def u2b64(unicode_string):
 
 def b642u(base64_bytestream):
     """
-    Convert base46 string to unicode.
+    Convert base64 string (in :py:data:`mini_buildd.setup.CHAR_ENCODING`) to unicode.
 
     >>> u = b642u('w5xuaWNvZGUgc3Ryw7xuZw==')
     >>> u.__class__.__name__
@@ -535,13 +539,6 @@ def b642u(base64_bytestream):
     return unicode(base64.b64decode(base64_bytestream), encoding=mini_buildd.setup.CHAR_ENCODING)
 
 
-def taint_env(taint):
-    env = os.environ.copy()
-    for name in taint:
-        env[name] = taint[name]
-    return env
-
-
 def get_cpus():
     try:
         return multiprocessing.cpu_count()
@@ -558,7 +555,9 @@ def list_get(list_, index, default=None):
 
 def mkdirs(path):
     """
-    .. note:: Needed for python 2.x only. For 3.x, just use 'exist_ok' parameter.
+    Make path and any parents needed. Succeed even if it does already exist in the first place.
+
+    .. note:: Needed for python 2.x only. For 3.x, just use 'exist_ok' parameter for os.makedirs().
     """
     try:
         os.makedirs(path)
@@ -570,118 +569,13 @@ def mkdirs(path):
             LOG.debug("Directory already exists, ignoring; {d}".format(d=path))
 
 
-def log_call_output(log, prefix, output):
-    output.seek(0)
-    for line in output:
-        log("{p}: {l}".format(p=prefix, l=line.decode(mini_buildd.setup.CHAR_ENCODING).rstrip('\n')))
-
-
-def args2shell(args):
+def rmdirs(path):
     """
-    Convenience: Convert an argument sequence to a command line maybe-suitable for cut and paste to a shell.
+    Remove path recursively. Succeed even if it does not exist in the first place.
     """
-    result = ""
-    for a in args:
-        if " " in a:
-            result += "\"" + a + "\""
-        else:
-            result += a
-        result += " "
-    return result
-
-
-def sose_call(args):
-    """
-    >>> sose_call(["echo", "-n", "hallo"])
-    u'hallo'
-    >>> sose_call(["ls", "__no_such_file__"])
-    Traceback (most recent call last):
-    ...
-    Exception: SoSe call failed (ret=2): ls __no_such_file__
-    """
-    result = tempfile.TemporaryFile()
-    ret = subprocess.call(args,
-                          stdout=result,
-                          stderr=subprocess.STDOUT)
-    if ret != 0:
-        log_call_output(LOG.error, "SoSe call failed", result)
-        raise Exception("SoSe call failed (ret={r}): {s}".format(r=ret, s=" ".join(args)))
-    result.seek(0)
-    return result.read().decode(mini_buildd.setup.CHAR_ENCODING)
-
-
-def call(args, run_as_root=False, value_on_error=None, log_output=True, error_log_on_fail=True, **kwargs):
-    """Wrapper around subprocess.call().
-
-    >>> call(["echo", "-n", "hallo"])
-    u'hallo'
-    >>> call(["id", "-syntax-error"], value_on_error="Kapott")
-    u'Kapott'
-    """
-
-    if run_as_root:
-        args = ["sudo", "-n"] + args
-
-    stdout = tempfile.TemporaryFile()
-    stderr = tempfile.TemporaryFile()
-
-    LOG.info("Calling: {a}".format(a=args2shell(args)))
-    try:
-        olog = LOG.debug
-        try:
-            subprocess.check_call(args, stdout=stdout, stderr=stderr, **kwargs)
-        except:
-            if error_log_on_fail:
-                olog = LOG.error
-            raise
-        finally:
-            try:
-                if log_output:
-                    log_call_output(olog, "Call stdout", stdout)
-                    log_call_output(olog, "Call stderr", stderr)
-            except Exception as e:
-                mini_buildd.setup.log_exception(LOG, "Output logging failed (char enc?)", e)
-    except:
-        if error_log_on_fail:
-            LOG.error("Call failed: {a}".format(a=" ".join(args)))
-        if value_on_error is not None:
-            return value_on_error
-        else:
-            raise
-    LOG.debug("Call successful: {a}".format(a=" ".join(args)))
-    stdout.seek(0)
-    return stdout.read().decode(mini_buildd.setup.CHAR_ENCODING)
-
-
-def call_sequence(calls, run_as_root=False, value_on_error=None, log_output=True, rollback_only=False, **kwargs):
-    """Run sequences of calls with rollback support.
-
-    >>> call_sequence([(["echo", "-n", "cmd0"], ["echo", "-n", "rollback cmd0"])])
-    >>> call_sequence([(["echo", "cmd0"], ["echo", "rollback cmd0"])], rollback_only=True)
-    """
-
-    def rollback(pos):
-        for i in range(pos, -1, -1):
-            if calls[i][1]:
-                call(calls[i][1], run_as_root=run_as_root, value_on_error="", log_output=log_output, **kwargs)
-            else:
-                LOG.debug("Skipping empty rollback call sequent {i}".format(i=i))
-
-    if rollback_only:
-        rollback(len(calls) - 1)
-    else:
-        i = 0
-        try:
-            for l in calls:
-                if l[0]:
-                    call(l[0], run_as_root=run_as_root, value_on_error=value_on_error, log_output=log_output, **kwargs)
-                else:
-                    LOG.debug("Skipping empty call sequent {i}".format(i=i))
-                i += 1
-        except:
-            LOG.error("Sequence failed at: {i} (rolling back)".format(i=i))
-            rollback(i)
-            raise
+    if os.path.exists(path):
+        shutil.rmtree(path)
+        LOG.info("Directory removed recursively: {p}".format(p=path))
 
 
 def tail(file_object, lines, line_chars=160):
@@ -884,22 +778,6 @@ def web_login(host, user, credentials,
         raise Exception("Login failed: {u}@{h}: {e}".format(u=user, h=host, e=e))
 
 
-SBUILD_KEYS_WORKAROUND_LOCK = threading.Lock()
-
-
-def sbuild_keys_workaround():
-    "Create sbuild's internal key if needed (sbuild needs this one-time call, but does not handle it itself)."
-    with SBUILD_KEYS_WORKAROUND_LOCK:
-        if os.path.exists("/var/lib/sbuild/apt-keys/sbuild-key.pub"):
-            LOG.debug("/var/lib/sbuild/apt-keys/sbuild-key.pub: Already exists, skipping")
-        else:
-            t = tempfile.mkdtemp()
-            LOG.warn("One-time generation of sbuild keys (may take some time)...")
-            call(["sbuild-update", "--keygen"], env=taint_env({"HOME": t}))
-            shutil.rmtree(t)
-            LOG.info("One-time generation of sbuild keys done")
-
-
 def clone_log(dst, src="mini_buildd"):
     "Setup logger named 'dst' with the same handlers and loglevel as the logger named 'src'."
     src_log = logging.getLogger(src)
