@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+from __future__ import absolute_import
 
 import copy
 import os
-import shutil
 import contextlib
 import glob
 import logging
@@ -14,6 +14,7 @@ import django.contrib.messages
 
 import mini_buildd.setup
 import mini_buildd.misc
+import mini_buildd.call
 
 import mini_buildd.models.base
 import mini_buildd.models.source
@@ -85,7 +86,7 @@ chroots (with <tt>qemu-user-static</tt> installed).
 
         @classmethod
         def mbd_host_architecture(cls):
-            return mini_buildd.misc.sose_call(["dpkg", "--print-architecture"]).strip()
+            return mini_buildd.models.source.Architecture.mbd_host_architecture()
 
         @classmethod
         def _mbd_get_supported_archs(cls, arch):
@@ -116,7 +117,7 @@ chroots (with <tt>qemu-user-static</tt> installed).
                                             f=self.mbd_get_backend().mbd_backend_flavor())
 
     def mbd_get_backend(self):
-        for cls, sub in {"filechroot": [], "dirchroot": [], "lvmchroot": ["looplvmchroot"]}.items():
+        for cls, sub in {"filechroot": [], "dirchroot": [], "lvmchroot": ["looplvmchroot"], "btrfssnapshotchroot": []}.items():
             if hasattr(self, cls):
                 c = getattr(self, cls)
                 for s in sub:
@@ -162,9 +163,9 @@ chroots (with <tt>qemu-user-static</tt> installed).
             (["/bin/mkdir", "--verbose", self.mbd_get_tmp_dir()],
              ["/bin/rm", "--recursive", "--one-file-system", "--force", self.mbd_get_tmp_dir()])] + self.mbd_get_backend().mbd_get_pre_sequence() + [
                  ([self.mbd_get_extra_option("Debootstrap-Command", "/usr/sbin/debootstrap"),
-                   "--variant=buildd",
-                   "--keyring={k}".format(k=self.mbd_get_keyring_file()),
-                   "--arch={a}".format(a=self.architecture.name),
+                   "--variant", "buildd",
+                   "--keyring", self.mbd_get_keyring_file(),
+                   "--arch", self.architecture.name,
                    self.source.codename,
                    self.mbd_get_tmp_dir(),
                    debootstrap_url],
@@ -173,7 +174,7 @@ chroots (with <tt>qemu-user-static</tt> installed).
                        ["/bin/rm", "--verbose", self.mbd_get_system_schroot_conf_file()])]
 
     def mbd_prepare(self, request):
-        mini_buildd.misc.mkdirs(os.path.join(self.mbd_get_path(), mini_buildd.setup.CHROOT_LIBDIR))
+        mini_buildd.misc.mkdirs(self.mbd_get_path())
 
         # Set personality
         self.personality = ""
@@ -208,25 +209,24 @@ personality={p}
                 gpg.add_pub_key(k.key)
             gpg.export(self.mbd_get_keyring_file())
 
-        mini_buildd.misc.call_sequence(self.mbd_get_sequence(), run_as_root=True)
+        mini_buildd.call.call_sequence(self.mbd_get_sequence(), run_as_root=True)
         MsgLog(LOG, request).info("{c}: Prepared on system for schroot.".format(c=self))
 
     def mbd_remove(self, request):
-        mini_buildd.misc.call_sequence(self.mbd_get_sequence(), rollback_only=True, run_as_root=True)
-        shutil.rmtree(self.mbd_get_path(),
-                      onerror=lambda f, p, e: MsgLog(LOG, request).warn("{c}: Failure removing data dir '{p}' (ignoring): {e}".format(c=self,
-                                                                                                                                      p=self.mbd_get_path(),
-                                                                                                                                      e=e)))
+        mini_buildd.call.call_sequence(self.mbd_get_sequence(), rollback_only=True, run_as_root=True)
+
+        mini_buildd.misc.rmdirs(self.mbd_get_path())
+
         MsgLog(LOG, request).info("{c}: Removed from system.".format(c=self))
 
     def mbd_sync(self, request):
         self._mbd_remove_and_prepare(request)
 
-    def _mbd_schroot_run(self, args, namespace="chroot", user="root"):
-        return mini_buildd.misc.sose_call(["/usr/bin/schroot",
-                                           "--chroot={n}:{c}".format(n=namespace, c=self.mbd_get_name()),
-                                           "--user={u}".format(u=user)] +
-                                          args)
+    def _mbd_schroot_run(self, call, namespace="chroot", user="root"):
+        return mini_buildd.call.Call(["/usr/bin/schroot",
+                                      "--chroot", "{n}:{c}".format(n=namespace, c=self.mbd_get_name()),
+                                      "--user", user] +
+                                     call).log().check().ustdout
 
     def mbd_check_sudo_workaround(self, request):
         """
@@ -237,7 +237,7 @@ personality={p}
         """
         has_sudo_workaround = False
         try:
-            self._mbd_schroot_run(["--directory=/", "--", "grep", "^{u}".format(u=os.getenv("USER")), "/etc/sudoers"])
+            self._mbd_schroot_run(["--directory", "/", "--", "grep", "^{u}".format(u=os.getenv("USER")), "/etc/sudoers"])
             has_sudo_workaround = True
         except:
             MsgLog(LOG, request).info("{c}: Ok, no sudo workaround found.".format(c=self))
@@ -246,7 +246,7 @@ personality={p}
             raise Exception("Chroot has sudo workaround (created with versions <= 1.0.4): Please run 'Remove' + 'PCA' on this chroot to re-create!")
 
     def mbd_backend_check(self, request):
-        "Subclasses may implement this to do extra backend checks."
+        "Subclasses may implement this to do extra backend-specific checks."
         MsgLog(LOG, request).info("{c}: No backend check implemented.".format(c=self))
 
     def mbd_check(self, request):
@@ -262,12 +262,17 @@ personality={p}
         # '/var/lib/schroot/mount/mini-buildd-wheezy-amd64-aaba77f3-4cba-423e-b34f-2b2bbb9789e1: device is busy.'
         # making this fail _and_ leave schroot cruft around.
         # Wtf! Hence we now just skip this ls test for now.
-        #  self._mbd_schroot_run(["--directory=/", "--", "/bin/ls"])
+        #  self._mbd_schroot_run(["--directory", "/", "--", "/bin/ls"])
 
         # Backend checks
         MsgLog(LOG, request).info("{c}: Running backend check.".format(c=self))
         self.mbd_get_backend().mbd_backend_check(request)
 
+        # Update base apt line (source might have switched to another archive).
+        apt_line = self.source.mbd_get_apt_line_raw(["main"])
+        self._mbd_schroot_run(["--", "/bin/sh", "-c", "echo '{apt_line}' >/etc/apt/sources.list".format(apt_line=apt_line)], namespace="source")
+        MsgLog(LOG, request).info("{c}: '/etc/apt/sources.list' updated: '{a}'.".format(c=self, a=apt_line))
+
         # "apt update/upgrade" check
         for args, fatal in [(["update"], True),
                             (["--ignore-missing", "dist-upgrade"], True),
@@ -276,7 +281,7 @@ personality={p}
             try:
                 MsgLog(LOG, request).info("=> Running: apt-get {args}:".format(args=" ".join(args)))
                 MsgLog(LOG, request).log_text(
-                    self._mbd_schroot_run(["--directory=/", "--", "/usr/bin/apt-get", "-q", "-o APT::Install-Recommends=false", "--yes"] + args,
+                    self._mbd_schroot_run(["--directory", "/", "--", "/usr/bin/apt-get", "-q", "-o", "APT::Install-Recommends=false", "--yes"] + args,
                                           namespace="source"))
             except:
                 MsgLog(LOG, request).warn("'apt-get {args}' not supported in this chroot.".format(args=" ".join(args)))
@@ -389,8 +394,8 @@ file={t}
         return [
             (["/bin/tar",
               "--create",
-              "--directory={d}".format(d=self.mbd_get_tmp_dir()),
-              "--file={f}".format(f=self.mbd_get_tar_file())] +
+              "--directory", self.mbd_get_tmp_dir(),
+              "--file", self.mbd_get_tar_file()] +
              self.TAR_ARGS[self.compression] +
              ["."],
              []),
@@ -438,13 +443,13 @@ lvm-snapshot-options=--size {s}G
 
     def mbd_get_pre_sequence(self):
         return [
-            (["/sbin/lvcreate", "--size={s}G".format(s=self.snapshot_size), "--name={n}".format(n=self.mbd_get_name()), self.mbd_get_volume_group()],
+            (["/sbin/lvcreate", "--size", "{s}G".format(s=self.snapshot_size), "--name", self.mbd_get_name(), self.mbd_get_volume_group()],
              ["/sbin/lvremove", "--verbose", "--force", self.mbd_get_lvm_device()]),
 
-            (["/sbin/mkfs", "-t{f}".format(f=self.filesystem), self.mbd_get_lvm_device()],
+            (["/sbin/mkfs", "-t", self.filesystem, self.mbd_get_lvm_device()],
              []),
 
-            (["/bin/mount", "-v", "-t{f}".format(f=self.filesystem), self.mbd_get_lvm_device(), self.mbd_get_tmp_dir()],
+            (["/bin/mount", "-v", "-t", self.filesystem, self.mbd_get_lvm_device(), self.mbd_get_tmp_dir()],
              ["/bin/umount", "-v", self.mbd_get_tmp_dir()])]
 
     def mbd_get_post_sequence(self):
@@ -452,8 +457,8 @@ lvm-snapshot-options=--size {s}G
 
     def mbd_backend_check(self, request):
         MsgLog(LOG, request).info("{c}: Running file system check...".format(c=self))
-        mini_buildd.misc.call(["/sbin/fsck", "-a", "-t{t}".format(t=self.filesystem), self.mbd_get_lvm_device()],
-                              run_as_root=True)
+        mini_buildd.call.Call(["/sbin/fsck", "-a", "-t", self.filesystem, self.mbd_get_lvm_device()],
+                              run_as_root=True).log().check()
 
 
 class LoopLVMChroot(LVMChroot):
@@ -486,7 +491,7 @@ class LoopLVMChroot(LVMChroot):
             if os.path.realpath(mini_buildd.misc.open_utf8(f).read().strip()) == os.path.realpath(self.mbd_get_backing_file()):
                 return "/dev/" + f.split("/")[3]
         LOG.debug("No existing loop device for {b}, searching for free device".format(b=self.mbd_get_backing_file()))
-        return mini_buildd.misc.call(["/sbin/losetup", "--find"], run_as_root=True).rstrip()
+        return mini_buildd.call.Call(["/sbin/losetup", "--find"], run_as_root=True).log().check().ustdout.rstrip()
 
     def mbd_get_pre_sequence(self):
         loop_device = self.mbd_get_loop_device()
@@ -506,3 +511,48 @@ class LoopLVMChroot(LVMChroot):
 
             (["/sbin/vgcreate", "--verbose", self.mbd_get_volume_group(), loop_device],
              ["/sbin/vgremove", "--verbose", "--force", self.mbd_get_volume_group()])] + super(LoopLVMChroot, self).mbd_get_pre_sequence()
+
+
+class BtrfsSnapshotChroot(Chroot):
+    """ Btrfs Snapshot chroot backend. """
+    class Meta(Chroot.Meta):
+        pass
+
+    class Admin(Chroot.Admin):
+        fieldsets = Chroot.Admin.fieldsets
+
+        @classmethod
+        def mbd_meta_add_base_sources(cls, msglog):
+            cls._mbd_meta_add_base_sources(BtrfsSnapshotChroot, msglog)
+
+    @classmethod
+    def mbd_backend_flavor(cls):
+        return "btrfs_snapshot"
+
+    def mbd_get_chroot_dir(self):
+        return os.path.join(self.mbd_get_path(), "source")
+
+    def mbd_get_snapshot_dir(self):
+        return os.path.join(self.mbd_get_path(), "snapshot")
+
+    def mbd_get_schroot_conf(self):
+        return """\
+type=btrfs-snapshot
+btrfs-source-subvolume={source}
+btrfs-snapshot-directory={snapshot}
+""".format(source=self.mbd_get_chroot_dir(),
+           snapshot=self.mbd_get_snapshot_dir())
+
+    def mbd_get_pre_sequence(self):
+        return [
+            (["/bin/btrfs", "subvolume", "create", self.mbd_get_chroot_dir()],
+             ["/bin/btrfs", "subvolume", "delete", self.mbd_get_chroot_dir()]),
+
+            (["/bin/mount", "-v", "-o", "bind", self.mbd_get_chroot_dir(), self.mbd_get_tmp_dir()],
+             ["/bin/umount", "-v", self.mbd_get_tmp_dir()]),
+
+            (["/bin/mkdir", "--verbose", self.mbd_get_snapshot_dir()],
+             ["/bin/rm", "--recursive", "--one-file-system", "--force", self.mbd_get_snapshot_dir()])]
+
+    def mbd_get_post_sequence(self):
+        return [(["/bin/umount", "-v", self.mbd_get_tmp_dir()], [])]
