@@ -2,6 +2,7 @@
 # django false-positives:
 # pylint: disable=unexpected-keyword-arg,no-value-for-parameter
 from __future__ import unicode_literals
+from __future__ import absolute_import
 
 import os
 import copy
@@ -635,9 +636,8 @@ pimp the internal priority up here.
         return result
 
     def mbd_get_sbuildrc_snippet(self, arch):
-        libdir = os.path.join(mini_buildd.setup.CHROOTS_DIR, self.base_source.codename, arch, mini_buildd.setup.CHROOT_LIBDIR)
         # Note: For some reason (python, django sqlite, browser?) the text field may be in DOS mode.
-        return mini_buildd.misc.fromdos(mini_buildd.misc.subst_placeholders(self.sbuildrc_snippet, {"LIBDIR": libdir}))
+        return mini_buildd.misc.fromdos(mini_buildd.misc.subst_placeholders(self.sbuildrc_snippet, {"LIBDIR": mini_buildd.misc.chroot_libdir_path(self.base_source.codename, arch)}))
 
     def mbd_get_reverse_dependencies(self):
         "When the distribution changes, all repos that use that distribution also change."
