@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+from __future__ import absolute_import
 
 import tempfile
 import urllib2
@@ -14,6 +15,8 @@ import django.contrib.messages
 
 import debian.deb822
 
+import mini_buildd.misc
+import mini_buildd.call
 import mini_buildd.gnupg
 
 import mini_buildd.models.base
@@ -164,7 +167,7 @@ class Architecture(mini_buildd.models.base.Model):
 
     @classmethod
     def mbd_host_architecture(cls):
-        return mini_buildd.misc.sose_call(["dpkg", "--print-architecture"]).strip()
+        return mini_buildd.call.Call(["dpkg", "--print-architecture"]).check().ustdout.strip()
 
     @classmethod
     def mbd_supported_architectures(cls, arch=None):
@@ -420,14 +423,17 @@ codeversion is only used for base sources.""")
         else:
             raise Exception("{s}: No archive found. Please add appropriate archive and/or check network setup.".format(s=self))
 
-    def mbd_get_apt_line(self, distribution, prefix="deb "):
-        allowed_components = [c.name for c in distribution.components.all()]
-        components = sorted([c for c in self.components.all() if c.name in allowed_components], cmp=cmp_components)
+    def mbd_get_apt_line_raw(self, components, prefix="deb "):
         return "{p}{u} {d} {c}".format(
             p=prefix,
             u=self.mbd_get_archive().url,
             d=self.codename + ("" if components else "/"),
-            c=" ".join([c.name for c in components]))
+            c=" ".join(components))
+
+    def mbd_get_apt_line(self, distribution, prefix="deb "):
+        allowed_components = [c.name for c in distribution.components.all()]
+        components = sorted([c for c in self.components.all() if c.name in allowed_components], cmp=cmp_components)
+        return self.mbd_get_apt_line_raw([c.name for c in components], prefix=prefix)
 
     def mbd_get_apt_pin(self):
         "Apt 'pin line' (for use in a apt 'preference' file)."
